@@ -1,7 +1,7 @@
 package Path::Router::Route::Match;
 use Moose;
 
-our $VERSION   = '0.04';
+our $VERSION   = '0.05';
 our $AUTHORITY = 'cpan:STEVAN';
 
 has 'path'    => (is => 'ro', isa => 'Str',     required => 1);
@@ -14,6 +14,8 @@ has 'route'   => (
     handles  => [qw[target]]
 );
 
+__PACKAGE__->meta->make_immutable;
+
 no Moose; 1;
 
 __END__
@@ -34,6 +36,8 @@ dispatching nessecary.
 
 =over 4
 
+=item B<new>
+
 =item B<path>
 
 This is the path that was matched.
